defmodule STBU.ClinicStateTest do
  use ExUnit.Case

  alias STBU.Interface.Clinic
  alias STBU.Interface.AppointmentScheduler, as: AS

  test "restart with previously scheduled appointment" do
    animal = %{name: "Fido", age: 30, type: :dog, beenVaccinated: true, picture: ""}
    clinic1 = Clinic.whereis()
    scheduler = Clinic.request_appointment(animal)

    appt = STBU.ClinicTest.select_randomly(scheduler)

    assert AS.finalize_details(scheduler) == :ok
    assert AS.book(scheduler) == :ok

    GenServer.stop(Clinic.whereis())
    Process.sleep(5)

    clinic2 = Clinic.whereis()
    assert is_pid(clinic2)
    assert Process.alive?(clinic2)
    assert clinic1 != clinic2

    scheduler2 = Clinic.request_appointment(animal)
    assert !appointment_available?(scheduler2, appt)
  end

  def appointment_available?(scheduler, {vet, week, day, time}) do
    if Enum.member?(AS.available_vets(scheduler), vet) do
      AS.select_vet(scheduler, vet)

      if Enum.member?(AS.available_weeks(scheduler), week) do
        AS.select_week(scheduler, week)

        if Enum.member?(AS.available_days(scheduler), day) do
          AS.select_day(scheduler, day)
          Enum.member?(AS.available_times(scheduler), time)
        else
          false
        end
      else
        false
      end
    else
      false
    end
  end

  test "clean up appointment scheduler state on exit" do
    animal = %{name: "Fido", age: 30, type: :dog, beenVaccinated: true, picture: ""}
    scheduler = Clinic.request_appointment(animal)
    appt = STBU.ClinicTest.select_randomly(scheduler)
    assert AS.finalize_details(scheduler) == :ok

    GenServer.stop(scheduler)
    Process.sleep(5)
    assert !Process.alive?(scheduler)

    scheduler2 = Clinic.request_appointment(animal)
    assert appointment_available?(scheduler2, appt)
  end
end
