defmodule STBU.ShelterStateTest do
  use ExUnit.Case

  alias STBU.Interface.{Shelter, PetViewer}

  test "restart with fewer pets after adoption" do
    shelter1 = Shelter.whereis()
    viewer = Shelter.view_pets()
    animal = STBU.ShelterTests.first_vaccinated(viewer)
    assert(PetViewer.try_play(viewer) == :ok)
    assert(PetViewer.try_adopt(viewer) == :ok)

    GenServer.stop(shelter1)
    Process.sleep(5)

    shelter2 = Shelter.whereis()
    assert is_pid(shelter2)
    assert Process.alive?(shelter2)
    assert shelter1 != shelter2

    pets = all_viewable(Shelter.view_pets())
    assert !Enum.member?(pets, animal)
  end

  def all_viewable(viewer, viewed \\ []) do
    case PetViewer.view_next(viewer) do
      nil -> viewed
      pet -> all_viewable(viewer, [pet | viewed])
    end
  end

  test "clean up pet viewer state on exit" do
    viewer = Shelter.view_pets()
    animal = PetViewer.view_next(viewer)
    assert(PetViewer.try_play(viewer) == :ok)
    GenServer.stop(viewer)
    Process.sleep(5)
    assert !Process.alive?(viewer)
    new_viewer = Shelter.view_pets()
    pets = all_viewable(new_viewer)

    assert Enum.member?(pets, animal),
           "#{inspect(animal)} expected to be in viewable pets:\n#{inspect(pets)}"
  end
end
